/*
SPDX-FileCopyrightText: Metadata Cleaner contributors
SPDX-License-Identifier: GPL-3.0-or-later
*/

@define-color accent_color #813d9c;
@define-color accent_bg_color @accent_color;
@define-color accent_fg_color #fff;

window.main {
	background: linear-gradient(180deg, @accent_bg_color 0px, @accent_bg_color 20px, @theme_bg_color 20px);
}

window.main headerbar {
	background-color: @accent_bg_color;
	color: @accent_fg_color;
}

button.help {
	transition: padding 0.1s, margin 0.1s, border-radius 0.1s;
}

button.help.small {
	border-radius: 0;
	padding-left: 18px;
	padding-right: 18px;
}

button.help:not(.small) {
	margin: 0 18px 18px;
}

listview.files>row {
	padding: 0;
	border-bottom: 1px solid @borders;
}

listview.files .remove,
listview.files .file {
	border-radius: unset;
	font-weight: unset;
}

listview.files .remove {
	padding: 6px 12px;
}

listview.files separator {
	opacity: 0;
	transition: 0.1s opacity;
}

listview.files>row:hover separator {
	opacity: 1;
	transition: none;
}

listview.metadata {
	background: none;
}

listview.metadata>row {
	margin: 12px 24px;
}

listview.metadata>row:last-child {
	margin-bottom: 24px;
}

row.metadata>box {
	margin: 6px;
	border-spacing: 12px;
}

.toolbar.details {
	background-color: @headerbar_bg_color;
	color: @headerbar_fg_color;
}

.badge {
	border-radius: 11px;
	background-color: rgba(0, 0, 0, 0.5);
	color: #fff;
	font-weight: bold;
	font-size: 12px;
}

.badge.flat {
	background: none;
	color: @theme_fg_color;
}

.badge label {
	margin: 0 6px;
}

.badge.metadata {
	background-color: @accent_color;
}

.badge.warning {
	background-color: @warning_color;
}

.badge.error {
	background-color: @error_color;
}

.badge.success {
	background-color: @success_color;
}

.badge label {
	padding: 0 0.4em;
}
