/*
SPDX-FileCopyrightText: Metadata Cleaner contributors
SPDX-License-Identifier: GPL-3.0-or-later
*/

@import './common.css';

window
{
	background: linear-gradient(180deg, @accent_bg_color 0px, @accent_bg_color 20px, @theme_bg_color 20px);
}

headerbar
{
	background: @accent_bg_color;
	color: @accent_fg_color;
}

.welcome-button
{
	border-radius: 0;
	padding: 10px 18px;
}

.file
{
	padding: 5px 10px 5px 6px;
	border-bottom: 1px solid @borders;
}

.file.selected
{
	background: alpha(currentColor, 0.1);
}

.metadata-view
{
	margin: 12px 24px;
	border-spacing: 6px;
}

.metadata-row
{
	margin: 6px;
	border-spacing: 12px;
}
