/*
SPDX-FileCopyrightText: Metadata Cleaner contributors
SPDX-License-Identifier: GPL-3.0-or-later
*/

@define-color accent_color #813d9c;
@define-color accent_bg_color @accent_color;
@define-color accent_fg_color #fff;


.badge
{
	border-radius: 11px;
	background-color: rgba(0, 0, 0, 0.5);
	color: #fff;
	font-weight: bold;
	font-size: 12px;
}

.badge.accent
{
	background-color: @accent_color;
}

.badge.success
{
	background-color: @success_color;
}

.badge label
{
	margin: 0 6px;
	padding: 0 0.4em;
}

.accent-bg
{
	background-color: @accent_bg_color;
	color: @accent_fg_color;
}
